#!/bin/bash

# A simple diagnostic script reporting outages in SPI communication
# unipi.technology @ 2021

# ================== DEFAULT VALUES ===================
# ==== CAN BE OVERRIDEN IN /etc/unipi/fw-diag-tool ====
CONFIG_FILE_PATH="/etc/unipi/fw-diag-tool.conf"
log_file="/opt/comm_outages.log"
REPORT_TO_SLACK='y' # Comment to disable
REPORT_TO_DB='y'  # Comment to disable
SUPRESS_SLACK_BOOTMSG=true
DISABLE_AUTOUPGRADE=false
#serial="/dev/ttyNS0" # Optional interconnection with DUT via an external RS485 line
SLACK_URL="https://hooks.slack.com/services/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" # Dead beef channel

#==== Internal variables ====
old_tick=0
counter=0
counter_reset=60
has_section_2=false
has_section_3=false
section_2_reported=false
section_3_reported=false
machine_name=$(hostname)

version_str="0.10" # Placeholder to be replaced in CI script by the right value

function remote_report_msg {
    #Remote log server
    if [[ -n $REPORT_TO_DB ]]; then
      msg=$(echo "${1}" | tr -d '\n')
      logger -P 514 -n 81.19.0.22 -d -t BRAIN -p 2 --rfc5424=notq "\'${msg}\'"
    fi

    test "${SUPRESS_SLACK_BOOTMSG}" = true && echo "${1}" | grep -q "BOOT" && return

    #Slack API
    if [[ -n $REPORT_TO_SLACK ]]; then
      machine_name=$(hostname)
      curl -X POST -H 'Content-type: application/json' --data "{\"text\": \" ${machine_name}(v.${version_str}): ${1}\n\"}" "$SLACK_URL"
    fi
}

function do_autoupgrade { # TODO - move to a separate dynamically-created executable file

    mount -o remount, rw /
    apt update
    available_version=$(apt-cache show fw-diag-tool --no-all-versions | grep -o "Version:[[:space:]][[:digit:]]*.[[:digit:]]*" | cut -d " " -f 2)

    if [ ! $version_str == $available_version ]; then
        remote_report_msg "Auto upgrade from $version_str to $available_version"
        apt install -y fw-diag-tool
        sleep 5
        exit 0
    fi
}

sleep 10

# Load configuration file
if [[ -f "${CONFIG_FILE_PATH}" ]]; then
    source "${CONFIG_FILE_PATH}"
fi

if [[ -n "${FORCE_MACHINE_NAME}" ]]; then
    machine_name="${FORCE_MACHINE_NAME}"
fi

if [ $(< /proc/uptime read a b && echo $((${a%.*}))) -le 60 ]; then
        echo "$(date "+%Y-%m-%d %H:%M:%S")  BOOT (fw-diag-tool v.${version_str})" >> $log_file
        remote_report_msg "BOOT (fw-diag-tool v.${version_str})"
fi

# Verify serial line
if [[ ! -c "${serial}" ]]; then
    serial=""
fi

# Verify 2 and 3 groups presence

mbpoll 127.0.0.1 -a 2 -0 -1 -r 1000 -q >/dev/null 2>&1 && has_section_2=true
mbpoll 127.0.0.1 -a 3 -0 -1 -r 1000 -q >/dev/null 2>&1 && has_section_3=true

while true; do
    # Check group 1 - BRAIN is always present
    if ! mbpoll 127.0.0.1 -a 1 -0 -1 -r 1000 -q >/dev/null 2>/dev/null; then
        echo "$(date "+%Y-%m-%d %H:%M:%S")" >> $log_file
        # Serial test is present
        if [ -n "${serial}" ]; then
            mb=$(mbpoll /dev/ttyNS0 -a 15 -P none -b 9600 -0 -r 1033 -c 13 -1 -q 2>&1)
            echo "$mb" >> $log_file
        fi

        echo "Counter: $counter" >> $log_file
        if [ "$counter" == "0" ]; then
            if [ -n "${serial}" ]; then
                remote_report_msg "${mb}"
            else
                remote_report_msg "SPI GROUP 1 OUTAGE (last brain tick ${tick})"
            fi
            counter=$counter_reset
        fi
        counter=$((counter-1))

    else
        # Catch watchdog reset flag
        if mbpoll 127.0.0.1 -a 1 -01 -r 1000 -q -t0 2>/dev/null | grep -q -e '\s1$'; then
            # reset watchdog bit
            mbpoll 127.0.0.1 -a 1 -01 -r 1000 -q -t0 1 2>/dev/null >/dev/null
            echo "$(date) Watchdog reboot" >> $log_file
            remote_report_msg "Watchdog RESET! (last brain tick ${tick})"
        fi

        # Catch firmware main loop counter reset
        tick=$(mbpoll "127.0.0.1" -a 1 -0 -t 4:int -r 1043 -c 1 -1 -q | grep -o '[[:digit:]]*$')
        tick=$(( $tick / 4  ))
        if [ $tick -lt $old_tick ]; then
            echo "TICK RESTART"
            echo "$(date) Tick counter mismatch (brain reset) - old tick: $old_tick; new tick: $tick!" >> $log_file
            remote_report_msg "Tick counter mismatch!"
        fi
        old_tick=$tick

        # reset counter, so next error will send message immediatelly
        counter=0
    fi

    # Check group 2 - capture the edge
    if [ $has_section_2 == true ] && ! mbpoll 127.0.0.1 -a 2 -0 -1 -r 1000 -q >/dev/null 2>/dev/null ; then
      if [ $section_2_reported == false ]; then
        echo "$(date) SPI GROUP 2 OUTAGE" >> $log_file
        remote_report_msg "SPI GROUP 2 OUTAGE"
        section_2_reported=true
      fi
    elif [ $section_2_reported == true ]; then
      echo "$(date) SPI GROUP 2 IS BACK" >> $log_file
      remote_report_msg "SPI GROUP 2 IS BACK"
      section_2_reported=false
    fi

    # Check group 3 - capture the edge
    if [ $has_section_3 == true ] && ! mbpoll 127.0.0.1 -a 3 -0 -1 -r 1000 -q >/dev/null 2>/dev/null ; then
      if [ $section_3_reported == false ]; then
        echo "$(date) SPI GROUP 3 OUTAGE" >> $log_file
        remote_report_msg "SPI GROUP 3 OUTAGE"
        section_3_reported=true
      fi
    elif [ $section_3_reported == true ]; then
      echo "$(date) SPI GROUP # IS BACK" >> $log_file
      remote_report_msg "SPI GROUP 3 IS BACK"
      section_3_reported=false
    fi

    sleep 1
done
